/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2009 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2007 Sun Microsystems, Inc.
 */
package org.netbeans.modules.python.editor.elements;

import java.util.Set;
import org.netbeans.modules.gsf.api.CompilationInfo;
import org.netbeans.modules.gsf.api.ElementKind;
import org.netbeans.modules.gsf.api.Modifier;
import org.netbeans.modules.python.editor.PythonAstUtils;
import org.netbeans.modules.python.editor.PythonStructureScanner;
import org.netbeans.modules.python.editor.scopes.SymbolTable;
import org.python.antlr.PythonTree;
import org.python.antlr.ast.Call;
import org.python.antlr.ast.ClassDef;
import org.python.antlr.ast.FunctionDef;
import org.python.antlr.ast.Name;

/**
 * Elements representing a node in a parse tree
 *
 * @author Tor Norbye
 */
public class AstElement extends Element {
    protected PythonTree node;
    protected String name;
    protected ElementKind kind;
    //protected CompilationInfo info;
    protected Set<Modifier> modifiers;
    protected SymbolTable scopes;

    public AstElement(SymbolTable scopes, PythonTree node, String name, ElementKind kind) {
        //this.info = info;
        this.scopes = scopes;
        this.node = node;
        this.name = name;
        this.kind = kind;
    }

    public static AstElement create(CompilationInfo info, PythonTree node) {
        SymbolTable scopes = PythonAstUtils.getParseResult(info).getSymbolTable();

        if (node instanceof FunctionDef) {
            return PythonStructureScanner.create(scopes, (FunctionDef)node);
        } else if (node instanceof ClassDef) {
            return PythonStructureScanner.create(scopes, (ClassDef)node);
        } else if (node instanceof Call) {
            String name = PythonAstUtils.getCallName((Call)node);
            return new AstElement(scopes, node, name, ElementKind.METHOD);
        } else if (node instanceof Name) {
            return new AstElement(scopes, node, ((Name)node).getInternalId(), ElementKind.VARIABLE);
        } else {
            return new AstElement(scopes, node, null, ElementKind.OTHER);
        }
    }

    public PythonTree getNode() {
        return node;
    }

    @Override
    public String getName() {
        return name;
    }

    @Override
    public ElementKind getKind() {
        return kind;
    }

    @Override
    public Set<Modifier> getModifiers() {
        if (modifiers == null) {
            if (name != null && scopes.isPrivate(node, name)) {
                modifiers = IndexedElement.PRIVATE_MODIFIERS;
            } else {
                modifiers = IndexedElement.PUBLIC_MODIFIERS;
            }
        }

        return modifiers;
    }

    public Object getSignature() {
        return name;
    }
}
